var lWarn = {logln:function (o){}};
var lInfo = {logln:function (o){}};

var lInfoFILE_NAME = "sr-config.log";
lWarn = new Logger(lInfoFILE_NAME);
lInfo = new Logger(lInfoFILE_NAME);

function storeSubParameter(path, value) {
    Property.setProperty('entries/' + path, value);
    Property.setFlag('entries/' + path, 'ARCHIVE', true);
}

function getNewIdValue() {
    var idValue = 0;
    var baseNode = Property.getNode('entries');
    if (baseNode == null) {
        return idValue;
    }
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        if (oChildNode.getChild('id') != null) {
            if (oChildNode.getChild('id').getValue() != null) {
                if (parseInt("" + oChildNode.getChild('id').getValue(), 10) >= idValue) {
                    idValue = parseInt("" + oChildNode.getChild('id').getValue(), 10) + 1;
                }
            }
        }
    }
    return idValue;
}

function isPropertyTreeNotCurrent(currentVersionString, compareVersionString) {
    var curVersion = currentVersionString.split('.'),
        comVersion = compareVersionString.split('.');
    for (var i=0; i < curVersion.length; i++) {
        if ((i < comVersion.length) &&
            (parseInt(curVersion[i]) < parseInt(comVersion[i]))) {
            return true;
        }
    }
    return false;
}

function convertOldTriggers() {
    var ndTriggers = Property.getNode('triggers');
    if (!ndTriggers) {
        return;
    }
    var triggers = ndTriggers.getChildren();
    var i;
    for (i = 0; i < triggers.length; ++i) {
        var ndOldParams = triggers[i].getChild('params'),
            oldParams,
            oldTrigger,
            oldActions;

        if (!ndOldParams) {
            lInfo.logln('Error: no old params found to convert on trigger ' + triggers[i].name);
            deleteOldTrigger(triggers[i]);
            continue;
        }

        try {
            // lInfo.logln('Debug: old params: ' + ndOldParams.getValue());
            oldParams = JSON.parse(ndOldParams.getValue());
            // lInfo.logln('Debug: old trigger: ' + decodeURIComponent(oldParams.trigger));
            oldTrigger = JSON.parse(oldParams.trigger);
            // lInfo.logln('Debug: old actions: ' + decodeURIComponent(oldParams.actions));
            oldActions = JSON.parse(oldParams.actions);
        } catch (e) {
            lInfo.logln('Deleting old trigger '+ triggers[i].name +': failed to parse');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        //
        // validity checks
        //

        if (!oldTrigger
            || isNaN(oldTrigger.scene)
            || isNaN(oldTrigger.zone)
                // allowed to be unset in very old versions (< 0.1.4)
            || (oldTrigger.group && isNaN(oldTrigger.group))
        ) {
            // delete invalid triggers
            lInfo.logln('Deleting old trigger '+ triggers[i].name +': missing trigger');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        if (!validScene(oldTrigger.scene)) {
            lInfo.logln('Deleting old trigger '+ triggers[i].name +': scene ' + oldTrigger.scene +' is not supported by this version');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        if (!oldActions
            || !oldActions.actions
            || !(oldActions.actions.length > 0)
        ) {
            // delete invalid triggers
            lInfo.logln('Deleting old trigger '+ triggers[i].name +': missing actions');
            deleteOldTrigger(triggers[i]);
            continue;
        }
        oldActions = oldActions.actions;

        //
        // Conversions
        //

        // new params object
        var params = {
            name: triggers[i].name, // or oldParams.name
            disabled: false, // there existed no possibility to disable an action
            scene: parseInt(oldTrigger.scene, 10),
            zone: parseInt(oldTrigger.zone, 10)
        };

        // Convert delay
        if (oldParams.duration === undefined) {
            params.delay = 0;
        } else if (isNaN(oldParams.duration)) {
            lInfo.logln('Warning: invalid delay on trigger '+ triggers[i].name +', assuming 0');
            params.delay = 0;
        } else {
            params.delay = 60 * parseInt(oldParams.duration, 10);
        }

        // Convert trigger group
        if (isNaN(oldTrigger.group)) {
            lInfo.logln('Warning: no trigger group is set on '+ triggers[i].name +': defaulting to group 1 (Lights)');
            params.group = 1;
        } else {
            params.group = parseInt(oldTrigger.group, 10);
        }

        // Convert old actions
        params.actions = [];
        var a;
        for (a = 0; a < oldActions.length; ++a) {
            var p = {};
            if (isNaN(oldActions[a].zone)
                || (isNaN(oldActions[a].scene) && isNaN(oldActions[a].value))
                || (oldActions[a].group && isNaN(oldActions[a].group))
                || (!isNaN(oldActions[a].value)
                    && (oldActions[a].value < 0 || oldActions[a].value > 255))
            ) {
                lInfo.logln('Error: no or invalid action set on trigger '+ triggers[i].name +' action at index '+ a +'. Skipping');
                continue;
            }
            p.zone = parseInt(oldActions[a].zone, 10);
            if (oldActions[a].group === undefined) {
                lInfo.logln('Warning: no group is set on '+ triggers[i].name +' action at index '+ a +', defaulting to group 1 (Lights)');
                p.group = 1;
            } else {
                p.group = parseInt(oldActions[a].group, 10);
            }
            if (oldActions[a].scene !== undefined) {
                if (!validScene(oldActions[a].scene)) {
                    lInfo.logln('Warning unsupported scene '
                            + oldActions[a].scene +' call from action '+ a
                            +' on '+ triggers[i].name +': skipping');
                    continue;
                }

                p.scene = parseInt(oldActions[a].scene, 10);
            } else if (oldActions[a].value !== undefined) {
                p.outputValue = parseInt(oldActions[a].value, 10);
            }
            params.actions.push(p);
        }
        if (params.actions.length === 0) {
            lInfo.logln('Error: no valid actions on trigger '+ triggers[i].name +': deleting');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        // store converted trigger
        deleteOldTrigger(triggers[i]);
        addTrigger(params);
        lInfo.logln('Converted trigger '+ params.idname);
    }
    Property.store();
} // convertOldTriggers

/**
 * Deletes an old (and converted but not saved) trigger from the property tree
 */
function deleteOldTrigger(ndTrigger) {
    ndTrigger.getParent().removeChild(ndTrigger);
    Property.store();
}

function updateData() {
    var baseNodes = Property.getNode('triggers');
    lInfo.logln("UpdateDataStart");
    if (baseNodes == null) {
        lInfo.logln("nothing to Update");
        return;
    }
    var fError=false;
    var fAnythingDone=false;
    var oBaseArray=baseNodes.getChildren();
    for ( var iIndex = 0; iIndex < oBaseArray.length; iIndex++) {
        fAnythingDone=true;
        lInfo.logln("found #" + iIndex);
        var oNewID=getNewIdValue();
        lInfo.logln("using id " + oNewID);
        try
        {
            var oTriggerNode=oBaseArray[iIndex];
            var sName=oTriggerNode.getChild('name').getValue();
            lInfo.logln("Name: " + sName);
            var fDisabled=oTriggerNode.getChild('disabled').getValue();
            lInfo.logln("fDisabled: " + fDisabled);
            storeSubParameter(oNewID + '/name', sName);
            storeSubParameter(oNewID + '/id', oNewID);
            storeSubParameter(oNewID + '/conditions/enabled', (fDisabled == true? false : true));

            if (oTriggerNode.getChild('delay')!=null) {
                var iDelay=oTriggerNode.getChild('delay').getValue();
                lInfo.logln("iDelay: " + iDelay);
                storeSubParameter(oNewID + '/delay', iDelay);
            }

            var triggerScene=oTriggerNode.getChild('scene').getValue();
            var triggerGroup=oTriggerNode.getChild('group').getValue();
            var triggerZone=oTriggerNode.getChild('zone').getValue();
            lInfo.logln("Triggering on : " + triggerZone + " " + triggerGroup + " " + triggerScene);

            var sSubpath=oNewID + '/triggers/0/';
            storeSubParameter(sSubpath + "type" ,'zone-scene');
            storeSubParameter(sSubpath + "zone" , triggerZone);
            storeSubParameter(sSubpath + "group" , triggerGroup);
            storeSubParameter(sSubpath + "scene" , triggerScene);
            storeSubParameter(sSubpath + "dsid" , -1);

            var oActionNodes=oTriggerNode.getChild('actions').getChildren();
            for (var jIndex=0;jIndex<oActionNodes.length;jIndex++) {
                var oActionNode=oActionNodes[jIndex];
                if (oActionNode.getChild('device')!=null) {
                    //device calls
                    if (oActionNode.getChild('blink')!=null) {
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('device').getValue() + " blink");
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'device-blink');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/dsid', oActionNode.getChild('device').getValue());
                    } else if (oActionNode.getChild('outputValue')!=null) {
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('device').getValue() + " outvalue " + oActionNode.getChild('outputValue').getValue);
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'device-value');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/dsid', oActionNode.getChild('device').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/value', oActionNode.getChild('outputValue').getValue());
                    } else if (oActionNode.getChild('scene')!=null){
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('device').getValue() + " scene " +oActionNode.getChild('scene').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'device-scene');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/dsid', oActionNode.getChild('device').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/scene', oActionNode.getChild('scene').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/force', false);
                    }
                } else {
                    // zone calls
                    if (oActionNode.getChild('blink')!=null) {
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('zone').getValue() + " " + oActionNode.getChild('group').getValue() + " blink");
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'zone-blink');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/zone', oActionNode.getChild('zone').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/group', oActionNode.getChild('group').getValue());
                    } else if (oActionNode.getChild('outputValue')!=null) {
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('zone').getValue() + " " + oActionNode.getChild('group').getValue() + " outvalue " + oActionNode.getChild('outputValue').getValue);
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'zone-value');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/zone', oActionNode.getChild('zone').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/group', oActionNode.getChild('group').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/value', oActionNode.getChild('outputValue').getValue());
                    } else if (oActionNode.getChild('scene')!=null){
                        lInfo.logln("Action #" + jIndex + " on : " + oActionNode.getChild('zone').getValue() + " " + oActionNode.getChild('group').getValue() +  " scene " +oActionNode.getChild('scene').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/type', 'zone-scene');
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/zone', oActionNode.getChild('zone').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/group', oActionNode.getChild('group').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/scene', oActionNode.getChild('scene').getValue());
                        storeSubParameter(oNewID + '/actions/' + jIndex + '/force', false);
                    }
                }
            }
            lInfo.logln("done. Delete old Node " + oTriggerNode.getPath());
            baseNodes.removeChild(oTriggerNode);
        }
        catch (e)
        {
            lWarn.logln("Fehler beim Datenupgrade " + e);
            if (Property.getNode('entries')!=null) {
                Property.getNode('entries').removeChild(oNewID);
            }
            fError=true;
        }
    }
    if (!fError) {
        lInfo.logln("All done. Delete old Node");
        Property.getNode('/scripts/scene-responder').removeChild('triggers');
    }
    if (fAnythingDone) {
        var evt = new Event("scene-responder.config", {actions:'updateTrigger'});
        evt.raise();
    }
    Property.store();
}

updateData();

